<?php
/**
 * Usepoll Plugin: allows to create simple polls
 *
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     Stephane Chazelas <stephane@artesyncp.com>
 *
 * heavily inspired (copy-pasted) from
 *             Esther Brunner <wikidesign@gmail.com>'s poll plugin
 */
 
if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../../').'/');
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'syntax.php');
 
/**
 * All DokuWiki plugins to extend the parser/rendering mechanism
 * need to inherit from this class
 */
class syntax_plugin_userpoll extends DokuWiki_Syntax_Plugin {
  
  /**
   * return some info
   */
  function getInfo(){
    return array(
      'author' => 'Stephane Chazelas',
      'email'  => 'stephane@artesyncp.com',
      'date'   => '2007-02-09',
      'name'   => 'Userpoll Plugin',
      'desc'   => 'allows to create simple polls',
      'url'    => 'http://wiki.splitbrain.org/plugin:userpoll',
    );
  }

  function getType(){ return 'substition';}
  function getPType(){ return 'block';}
  function getSort(){ return 167; }
  
  /**
   * Connect pattern to lexer
   */
  function connectTo($mode){
    $this->Lexer->addSpecialPattern('<userpoll.*?>.+?</userpoll>', $mode, 'plugin_userpoll');
  }

  /**
   * Handle the match
   */
  function handle($match, $state, $pos, &$handler){
    $match = substr($match, 10, -11);  // strip markup
    list($title, $options) = preg_split('/>/u', $match, 2);
    if (!$options){
      $options = $title;
      $title   = NULL;
    }
    $options = explode('*', $options);
    
    $c = count($options);
    for ($i = 0; $i < $c; $i++){
      $options[$i] = trim($options[$i]);
    }
        
    return array(trim($title), $options);
  }

  /**
   * Create output
   */
  function render($mode, &$renderer, $data) {
  
    if ($mode == 'xhtml'){
      global $ID;
      
      $options = $data[1];
      $title   = $renderer->_xmlEntities($data[0]);
      
      // prevent caching to ensure the poll results are fresh
      $renderer->info['cache'] = false;
            
      // get poll file contents
      $pollid = md5('@userpoll@'.$title);
      $pfile = metaFN($pollid, '.userpoll');
      $poll  = unserialize(@file_get_contents($pfile));
      
      // output the poll
      $renderer->doc .= '<fieldset class="userpoll">'.
        '<legend>'.$title.'</legend>';
      $more = trim(array_shift($options));
      if ($more){
        $renderer->doc .= '<div>'.$renderer->_xmlEntities($more).'</div>';
      }
        
      if (isset($_SERVER['REMOTE_USER'])) {
        $user = $_SERVER['REMOTE_USER'];
        if (isset($poll['users']) && isset($poll['users'][$user])) {
          if (($vote = $_REQUEST['vote']) == '@FORGET@' && $pollid == $_REQUEST['pollid']) {
            // user changed his/her mind, wipe him from the records
            $opt = $poll['users'][$user];
            unset($poll['users'][$user]);
            $poll['results'][$opt] -= 1;
            $poll['votes'] -= 1;
            $fh = fopen($pfile, 'w');
            fwrite($fh, serialize($poll));
            fclose($fh);
            $renderer->doc .= $this->_pollForm($options, $renderer, $pollid);
          } else {
            // display results
            $renderer->doc .= $this->_pollResults($poll, $pollid);
          }

        } elseif (($vote = $_REQUEST['vote']) && $pollid == $_REQUEST['pollid']){
        
          // user has just voted -> update results
          $c = count($options);
          for ($i = 0; $i < $c; $i++){
            $opt = $renderer->_xmlEntities($options[$i]);
            if ($vote == $opt){
              $poll['results'][$opt] += 1;
              $poll['votes'] += 1;
              $poll['users'][$user] = $opt;
            } elseif (!isset($poll['results'][$opt])){
              $poll['results'][$opt] = 0;
            }
          }
          $fh = fopen($pfile, 'w');
          fwrite($fh, serialize($poll));
          fclose($fh);
          
          // display results
          $renderer->doc .= $this->_pollResults($poll, $pollid);
        } elseif (count($options) > 0){
        
          // display poll form
          $renderer->doc .= $this->_pollForm($options, $renderer, $pollid);

        } else {
          
          // display results
          $renderer->doc .= $this->_pollResults($poll);

        }
      } else {
        $renderer->doc .= '<div class="userpoll_error">' .$this->getLang('login_required') . '</div>'
          . $this->_pollResults($poll);
      }
      $renderer->doc .= '</fieldset>';
          
      return true;
    }
    return false;
  }
  
  function _pollResults($poll, $pollid = false){
    global $ID;
    global $lang;

    $total = $poll['votes'];
    if ($total == 0) return '';
  
    $ret = '<table class="blind">';
    $c = count($poll['results']);
    $options = array_keys($poll['results']);
    $votes   = array_values($poll['results']);
    for ($i = 0; $i < $c; $i++){
      $absolute = $votes[$i];
      $percent  = round(($absolute*100)/$total);
      $ret .= '<tr><td>'.$options[$i].'</td><td><div class="userpoll_bar">';
      if ($percent) $ret .= '<div class="userpoll_full" style="width:'.($percent*2).'px">&nbsp;</div>';
      $ret .= '</div></td><td class="rightalign">'.$percent.'%</td>'.
        '<td class="rightalign">('.$absolute.')</td></tr>';
    }
    if ($pollid) {
      // user had already voted, allow him/her to changer his/her mind
      $ret .= '<tr><td colspan="4">'
        . '<form id="userpoll__form" method="post" action="'.script().'" accept-charset="'.$lang['encoding'].'">'
        . '<div class="no"><input type="hidden" name="do" value="show" />'
        . '<input type="hidden" name="pollid" value="' . $pollid . '" />'
        . '<input type="hidden" name="id" value="'.$ID.'" />'
        . '<input type="hidden" name="vote" value="@FORGET@">'
        . '<input class="button" type="submit" value="' . $this->getLang('btn_changemind') . '"></form></div></td></tr>';
    }
    $ret .= '</table>';
    
    return $ret;
  }
  
  function _pollForm($options, &$renderer, $pollid){
    global $lang;
    global $ID;
    
    $i = 0;
    $ret = '<form id="userpoll__form" method="post" action="'.script().'" accept-charset="'.$lang['encoding'].'"><div class="no">'.
      '<input type="hidden" name="do" value="show" />'.
      '<input type="hidden" name="pollid" value="' . $pollid . '" />'.
      '<input type="hidden" name="id" value="'.$ID.'" />';
    foreach ($options as $option){
      $i++;
      $option = $renderer->_xmlEntities($option);
      $ret.= '<label class="simple" for="userpoll__option'.$i.'">'.
        '<input type="radio" name="vote" id="userpoll__option'.$i.'" '.
        'value="'.$option.'" /> <span>'.$option.'</span></label>';
    }
    $ret .= '<input class="button" type="submit" '.
      'value="'.$this->getLang('btn_vote').'" />'.
      '</div></form>';
    
    return $ret;
  }
}
 
//Setup VIM: ex: et ts=4 enc=utf-8 :
